local CIR = _G.CooldownIconsR
local icons = CIR.icons
local f
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)

local ScriptEnv = { _G = _G, print = print }

CIR.ScriptEnv = ScriptEnv

local UnitPower4 = {
	"UNIT_MANA",
	"UNIT_RAGE",
	"UNIT_ENERGY",
	"UNIT_RUNIC_POWER",	
}

function CIR:RegisterObjForEvent(obj,event)
	if not self.events[event] then
		self.events[event] = { }
		if LCI:inTable(event,UnitPower4) then
			self:RegisterEvent(event,"UNIT_POWER")
		else
			self:RegisterEvent(event)
		end
	end
	--print(LCI:inTable(obj.name,self.events[event]))
	if not LCI:inTable(obj.name,self.events[event]) then
		self.events[event][#self.events[event]+1]=obj.name
	end
end

local function compile_script(script)
	local func = assert(loadstring(script))
	setfenv(func, ScriptEnv)
	func()
end

local function spell(name)
	local id
	if f then
		--user input was id
		if type(name) == number then
			id = name
			name = GetSpellInfo(name)
		end
		f:SetSpell(name)
		
	end
end
ScriptEnv.spell = spell

local function item(name)
	local id
	if f then
		if type(name) == number then
			id = name
			name = GetItemInfo(name)
		end
		f:SetItem(name)
	end
end
ScriptEnv.item = item

local valid_flags ={
	equip={
		[0]="ammo",
		[1]="head",
		[2]="neck",
		[3]="shoulder",
		[4]="shirt",
		[5]="chest",
		[6]="belt",
		[7]="legs",
		[8]="feet",
		[9]="wrist",
		[10]="gloves",
		[11]="finger1",
		[12]="finger2",
		[13]="trinket1",
		[14]="trinket2",
		[15]="back",
		[16]="mainhand",
		[17]="offhand",
		[18]="ranged",
		[19]="tabard",
	},
}

local function flag(name)
	if f then
		local isFlag, id = LCI:inTable(name,valid_flags.equip)
		if not isFlag then
			error("Invalid flag, valid flags are: "..tconcat(valid_flags.equip,", "))
			return false
		else
			f:SetItem(GetInventoryItemLink(id))
			CIR:RegisterObjForEvent(f,"UNIT_INVENTORY_CHANGED")
		end
	end
end

--returns secondary unit ressources (required for 4.0)
local function UnitRessources(unit)
	_, class = UnitClass(unit)
	local sRessources = {
		["PALADIN"] = 9,
		["DEATHKNIGHT"] = 5,
	}
	return 
end

--Conditionals
--PlayerPower
--returns primary power (energy - cat, etc.)
local function PlayerPower()
	CIR:RegisterObjForEvent(f,"UNIT_"..select(2,UnitPowerType("player")))
	return UnitPower("player")
end
ScriptEnv.PlayerPower = PlayerPower
--TargetPower
local function TargetPower()
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	CIR:RegisterObjForEvent(f,"UNIT_"..select(2,UnitPowerType("target")))
	return UnitPower("target")
end
ScriptEnv.TargetPower = TargetPower
--PlayerBuff
local function PlayerBuff(name,stack)
	CIR:RegisterObjForEvent(f,"UNIT_AURA")
	if stack == nil then
		stack = 0
	end
	n,_,_,c = UnitAura("player",name)
	return (n==name and c==stack)
end
ScriptEnv.PlayerBuff = PlayerBuff
--TargetBuff
local function TargetBuff(name,stack)
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	CIR:RegisterObjForEvent(f,"UNIT_AURA")	
	if stack==nil then
		stack = 0
	end
	n,_,_,c = UnitAura("target",name)
	return (n==name and c==stack)
end
ScriptEnv.TargetBuff = TargetBuff
--TargetHP
local function TargetHP()
	CIR:RegisterObjForEvent(f,"UNIT_HEALTH")
	return UnitHealth("target")/UnitHealthMax("target")
end
ScriptEnv.TargetHP = TargetHP
local function SpellOnCooldown(name)
	CIR:RegisterObjForEvent(f,"SPELL_UPDATE_COOLDOWN")
	local _,duration = GetSpellCooldown(name)
	if duration and duration>2 then
		return true
	else
		return false
	end
end
ScriptEnv.SpellOnCooldown = SpellOnCooldown

--icon controls
local function GrayscaleOn()
	f:GrayscaleOn()
end
ScriptEnv.GrayscaleOn = GrayscaleOn
local function GrayscaleOff()
	f:GrayscaleOff()
end
ScriptEnv.GrayscaleOff = GrayscaleOff
local function ResetGrayscale()
	f:ResetGrayscale()
end
ScriptEnv.ResetGrayscale = ResetGrayscale
local function RangeOn()
	f:RangeColoringOn()
end
ScriptEnv.RangeOn = RangeOn
local function RangeOff()
	f:RangeColoringOff()
end
ScriptEnv.RangeOff = RangeOff
local function ResetRange()
	f:ResetRange()
end
ScriptEnv.ResetRange = ResetRange

--
local function TalentGroup()
	CIR:RegisterObjForEvent(f,"ACTIVE_TALENT_GROUP_CHANGED")
	return GetActiveTalentGroup()
end
local function PlayerStance()
	CIR:RegisterObjForEvent(f,"UPDATE_SHAPESHIFT_FORM")
	return GetShapeshiftForm()
end
ScriptEnv.TalentGroup = TalentGroup
ScriptEnv.PlayerStance = PlayerStance



function CIR:ProcessScriptForIcon(script,obj)
	f = nil
	f = obj
	compile_script(script)
end
