local CIR = CooldownIconsR
local options = CIR.Options
local media = LibStub("LibSharedMedia-3.0")
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)


local function iconConfigHandler(obj,th,v)
	local icons = CIR.icons
	local db = CIR.db.char.icondb
	if th=='iZoom' then
		local mZoom = LCI:tonumber(v) *.08
		icons[obj].texture:SetTexCoord(0.0+mZoom, 1.0-mZoom, 0.0+mZoom, 1.0-mZoom)
	elseif th=='iRange' then
		if not v then
			icons[obj]:RangeColoringOff()
		else
			icons[obj]:ResetRange()
		end
	elseif th=='iGrayscale' then
		if not v then
			icons[obj]:GrayscaleOff()
		else
			icons[obj]:ResetGray()
		end
	elseif th=='iIconSize' then
		icons[obj]:SetWidth(v)
		icons[obj]:SetHeight(v)
		iconConfigHandler(obj,'iZoom',1)
		if LBF then
			LBF:Group('CooldownIcons R'):AddButton( icons[obj], {Icon=icons[obj].texture})
		end
	elseif th=='iX' then
		icons[obj]:SetPoint("CENTER",v,icons[obj]:GetY())
	elseif th=='iY' then
		icons[obj]:SetPoint("CENTER",icons[obj]:GetX(),v)
	elseif th=='script' then
		CIR:ProcessScriptForIcon(v,icons[obj])
	elseif th=='iDisplayName' then
		if v=="" then
			v=obj
		end
		CIR:add_icon_settings(obj,v)
	end
end

function CIR:getIconValue(info)
	return CIR.db.char.icondb[info[#info-2]][info[#info]]
end
function CIR:setIconValue(info,value)
	local obj,th = info[#info-2],info[#info]
	CIR.db.char.icondb[obj][th] = value
	iconConfigHandler(obj,th,value)
end

local iconConfigTable=
function()
	return {
		iIcon={
			type = 'group',
			name = "General",
			order = 1,
			args={
				iDisplayName={
					type='input', name='Name',order=1,
					desc='Changes the name of the icon.',
					get='getIconValue',
					set='setIconValue',width='full',
				},
				iAccuracy={
					type='range', name='Amount of fractional digits',
					desc='Enables fractional digits to keep track of real Cooldowns (ex. Chain Lightning 3.5 cooldown - without it would be rounded to 3)\n0=disable, 1=1 digit after comma(,)\nCurrently only enabled if used Timeformat: 1,3 and 4\nNote: This will also lower/higher the updateFrequency.',
					min=0,max=2,
					step=1,
					get='getIconValue',
					set='setIconValue',
					disabled=true,
				},
				iZoom={
					type='toggle', name=' Toggle Zoom',--order=103,
					desc='Enables/Disables zoom for icons.',
					get="getIconValue",
					set="setIconValue",
				},
				iRange={
					type='toggle', name=' Toggle Range',--order=103,
					desc='Enables/Disables Range-Check',
					get="getIconValue",
					set="setIconValue",
				},
				iGrayscale={
					type='toggle', name=' Toggle Grayscale',--order=103,
					desc='Enables/Disables Grayscale',
					get="getIconValue",
					set="setIconValue",
				},
				iIconSize={
					type='range', name=' Icon Size',
					desc='Changes the size of icons.',
					min=2,max=124,width='full',
					get="getIconValue",
					set="setIconValue",			
				},
				iX = {
					type='range', name=' X Coordinate',
					desc='Changes the x location on screen.',
					min=-floor(UIParent:GetWidth()/2),max=floor(UIParent:GetWidth()/2),step=1,order=-2,width="double",
					get='getIconValue',
					set='setIconValue',
				},
				iY = {
					type='range', name=' Y Coordinate',
					desc='Changes the Y location on screen.',
					min=-floor(UIParent:GetHeight()/2),max=floor(UIParent:GetHeight()/2),step=1,order=-1,width="double",
					get='getIconValue',
					set='setIconValue',
				},
			}
		},
		iFont = {
			type = 'group',
			name = "Font",
			order = 2,
			args={
				iFontSize = {
						type='range', name=' Font size',--order=112,
						desc='Changes the size of the font.',
						min=1,max=32,width='full',
						step=1,
						get='getIconValue',
						set='setIconValue',
				},
				iFontStyle={
					type='select',name=' Font',
					values=media:List('font'), style='dropdown',
					get="getFontStyle",
					set="setFontStyle",
				},
				iFontOffsetX={
					type='range', name=' Font X-Offset',--order=112,
					desc='Changes the x-offset of the font to the center of the icon.',
					min=1,max=32,width='full',
					step=1,
					get='getIconValue',
					set='setIconValue',
				},
				iFontOffsetY={
					type='range', name=' Font Y-Offset',--order=112,
					desc='Changes the y-offset of the font to the center of the icon.',
					min=1,max=32,width='full',
					step=1,
					get='getIconValue',
					set='setIconValue',
				},
				iFontColor={
					type='color', name=' Text Color',order=2,
					desc='Changes the color of the Cooldown Text displayed on Icon.',
					hasAlpha=true,
					get='getTextColor',
					set='setTextColor',
				},
			}
		},
		iScript = {
			type='group',
			name='Script',
			order = 3,
			args = {
				script= {
					type='input',
					name='',
					width='full',
					get='getIconValue',
					set='setIconValue',
					multiline=24,
				},
			},
		},
	}
end


local function addNewBtn_Func(info)
		--init value
		local db = CIR.db
		local icondb = db.char.icondb
		if not(type(icondb)=='table') then
			print("Create IconDB")
			CIR.db.char.icondb={}
		end
		ButtonID = LCI:Getn(CIR.db.char.icondb)
		print(ButtonID)
		local newIcon = "Icon"..ButtonID
		local a_config = {
			name=newIcon,
			type='group',
			width='double',
			cmdHidden=true,
			childGroups='tab',
			args=iconConfigTable(),
		}
		local a_db = {
			id = ButtonID,
			script = [[ ]],
			--font
			iFontSize = db.char.gFont.gFontSize,
			iFontStyle = db.char.gFont.gFontStyle,
			iFontOffsetX = db.char.gFont.gFontOffsetX,
			iFontOffsetY = db.char.gFont.gFontOffsetY,
			iFontColor = {
			db.char.gFont.gFontColor[1],
			db.char.gFont.gFontColor[2],
			db.char.gFont.gFontColor[3],
			db.char.gFont.gFontColor[4],
			},
			--icon
			iZoom = db.char.gIcon.gZoom,
			iRange = db.char.gIcon.gRange,
			iGrayscale = db.char.gIcon.gGrayscale,
			iIconSize = db.char.gIcon.gIconSize,
			iAccuracy = db.char.gIcon.gAccuracy,
			iDisplayType = db.char.gIcon.gDisplayType,
			iTimeFormat = db.char.gIcon.gTimeFormat,
			iX = 0,
			iY = 0,
			iDisplayName = newIcon,
		}
		--SV update
		CIR.db.char.icondb[newIcon] = a_db
		CIR.Options.args.icon_settings.args[newIcon] = a_config
		CIR:CreateIcon(newIcon)
end

CIR.add_icon_settings=
function(self,icon,iDN)
	if iDN=="" then iDN = icon end
	self.Options.args.icon_settings.args[icon] = {
		name=iDN or icon,
		type='group',
		cmdHidden=true,
		width='double',
		childGroups='tab',
		args=iconConfigTable(),
	}
end

CIR.get_icon_settings=
function()
	return {
		addNewBtn={
			type='execute',
			name="Add new Icon",
			desc="Adds a new customizeable Icon for a new Cooldown to be tracked.",
			func=addNewBtn_Func,
			order=-1,
		},
	}
end

