--[[
	##AddOn	  : CooldownIcons R(evamped)
	##Subfile : icon.lua
	##Author  : Hati ( xhatix @ curseforge.com )
	Containing functions used by icons through ScriptEnv/Sandbox
--]]
local CIR = CooldownIconsR
local media = LibStub("LibSharedMedia-3.0")
local LBF = LibStub("LibButtonFacade", true)
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)
CIR.icons = { }
local dftl_icon
local texture_null = "Interface\\Icons\\INV_Misc_QuestionMark"
local format = format

--forced range
local function RangeColoringOn(self)
	self.texture:SetVertexColor(1,0,0,1)
	self.forcedrange = true
end
local function RangeColoringOff(self)
	self.texture:SetVertexColor(1,1,1,1)
	self.forcedrange = true
end
local function ResetRange(self)
	self.forcedrange = false
end
--forced unable
local function GrayscaleOn(self)
	self.texture:SetDesaturated(1)
	self.forcedgray = true
end
local function GrayscaleOff(self)
	self.texture:SetDesaturated(0)
	self.forcedgray = true
end
local function ResetGray(self)
	self.forcedgray = false
end
--geometric
local function GetX(self)
	return select(4,self:GetPoint())
end
local function GetY(self)
	return select(5,self:GetPoint())
end
local function GetLeft(self)
	return (self:GetX()-self:GetWidth()/2)
end
local function GetRight(self)
	return (self:GetX()+self:GetWidth()/2)
end
local function GetTop(self)
	return (self:GetY()+self:GetHeigth()/2)
end
local function GetBottom(self)
	return (self:GetY()-self:GetHeigth()/2)
end
--shortcuts
local function SetSpell(self,name)
	self.texture:SetTexture(select(3,GetSpellInfo(name)))
	self.watch = name
	self.type = 'spell'
end
local function SetItem(self,name)
	self.texture:SetTexture(select(10,GetItemInfo(name)))
	self.watch = name
	self.type = 'item'
end
local function Clear(self)
	
end


--timer
local function SetTime(self,start,duration)
	local now = GetTime()
	racc = CIR.db.char.gIcon.gAccuracy or 1
	local t = (duration-(now-start))
	t = format("%."..racc.."f",t)
	self.text:SetText(t)
	if not self.forcedgray then
		self.texture:SetDesaturated(1)
	end
end
--

function CIR:CreateIcon(name)
	local db = CIR.db.char.icondb
	CIR.icons[name] = { }
	local icon = CIR.icons[name]
	--button type to support LBF--
	icon = CreateFrame("Button", "CIR_"..name, UIParent)
	icon.name = name
	icon:SetPoint("Center",db[name].iX,db[name].iY)
	icon:SetWidth(db[name].iIconSize)
	icon:SetHeight(db[name].iIconSize)
	icon:SetMovable(false)
	icon:EnableMouse(false)
	icon:SetScript("OnMouseDown",nil)
	icon:SetScript("OnMouseUp",nil)
	--texture
	icon.texture = icon:CreateTexture(icon:GetName().."_Texture")
	icon.texture:SetAllPoints(icon)
	icon.texture:SetTexture(texture_null)
	--text
	icon.text = icon:CreateFontString(icon:GetName().."_Text","ARTWORK")
	icon.text:SetPoint("CENTER",icon,"CENTER",db[name].iFontOffsetX,db[name].iFontOffsetY)
	icon.text:SetFont(media:Fetch("font",media:List('font')[db[name].iFontStyle] or "Fonts\\FRIZQT__.TTF"),db[name].iFontSize,"OUTLINE")
	icon.text:SetTextColor(unpack(db[name].iFontColor or {1.0,1.0,1.0,1.0}))
	
	--some variables
	icon.displayType = db[name].iDisplayType or 1
	icon.timeFormat = db[name].iTimeFormat or 1
	
	--add functions
	icon.RangeColoringOn = RangeColoringOn
	icon.RangeColoringOff = RangeColoringOff
	icon.ResetRange = ResetRange
	icon.GrayscaleOn = GrayscaleOn
	icon.GrayscaleOff = GrayscaleOff
	icon.ResetGray = ResetGray
	icon.GetX = GetX
	icon.GetY = GetY
	icon.GetLeft = GetLeft
	icon.GetRight = GetRight
	icon.GetTop = GetTop
	icon.GetBottom = GetBottom
	icon.SetSpell = SetSpell
	icon.SetItem = SetItem
	icon.SetTime = SetTime
	
	if LBF then
			LBF:Group('CooldownIcons R'):AddButton( icon, {Icon=icon.texture})
	end
	local mZoom = LCI:tonumber(db[name].iZoom) *.08
	icon.texture:SetTexCoord(0.0+mZoom, 1.0-mZoom, 0.0+mZoom, 1.0-mZoom)
	
	
	CIR.icons[name] = icon
end







